//
//  Created by Lukas Hlavaty on 04.01.17.
//  Copyright (c) 2017 Lukas Hlavaty. All rights reserved.
//
#include "GameEngine.h"

GameEngine::GameEngine()
{
    m_PL= new Player();
    m_AI= new Player();
    m_Analyzer= new VKostkyAnalyzer();
    m_Statistika = new Statistika();
    m_GUI = new GUI();
    m_docasneSkore= 0;

    inicializace(10000, 350);

    for (int i=0; i<6; i++){
        m_posledniHod[i].hodnota= 0;
        m_posledniHod[i].poradi= 10;
    }
}


GameEngine::~GameEngine()
{
    delete(m_PL);
    delete(m_AI);
    delete(m_Analyzer);
    delete (m_Statistika);
    delete(m_GUI);
}

void GameEngine::gameReset()
{
    delete(m_PL);
    delete(m_AI);
    delete(m_Analyzer);
    delete (m_Statistika);
    delete(m_GUI);

    m_PL= new Player();
    m_AI= new Player();
    m_Analyzer= new VKostkyAnalyzer();
    m_Statistika = new Statistika();
    m_GUI = new GUI();
    m_docasneSkore= 0;
    m_odlozeneCisla.clear();
}

void GameEngine::inicializace(int endScore, int minScore){
    m_EndScore= endScore;
    m_MinScore= minScore;
}

void GameEngine::play(bool newGame){
    if (newGame)
        m_GUI->uvitani();

    bool konec= false;
    bool hrajeHrac= true;
    while(!konec)
    {
        if (hrajeHrac)
        {
           fazeHazeni(false);
           hrajeHrac= false;
        }
        else
        {
            fazeHazeni(true);
            hrajeHrac= true;
        }

        if (vyhra())
            konec= true;
    }

    system("cls");

    if (m_PL->get_stavSkore()>= m_EndScore)
        m_GUI->vyhra(true, m_PL, m_AI);
    else
        m_GUI->vyhra(false, m_PL, m_AI);

    m_GUI->credits();
}



void GameEngine::naplnPolePosledniHod(Player* hrac){
    for (int i=0; i<6; i++){
        m_posledniHod[i].hodnota= hrac->vratPosledniHodKostky(i);
        m_posledniHod[i].poradi= i;
    }
    bubleSort();
}

void GameEngine::bubleSort(){
    for (int i=0; i<6; i++){
        for (int j=0; j<5; j++){
            if(m_posledniHod[j].hodnota>m_posledniHod[j+1].hodnota){
                sKostkaPoradi pom= m_posledniHod[j+1];
                m_posledniHod[j+1]= m_posledniHod[j];
                m_posledniHod[j]= pom;
            }
        }
    }
}

void GameEngine::vypisPolePosledniHod(){
    std::cout<<"Vypis pole :"<<std::endl;
    for (int i=0; i<6; i++)
        std::cout<<m_posledniHod[i].hodnota<<" ";
    //std::cout<<std::endl;

}

void GameEngine::vypisNalezenychKombinaci(){
    if(m_Analyzer->vratNalezeneKombinace().size()>0){
        int soucetBodu= 0;
        std::cout<<std::endl;
        for (int i=0; i<m_Analyzer->vratNalezeneKombinace().size(); i++){
                //std::cout<<"Kombinace "<<i+1<<":"<<std::endl;
                for (int j=0; j<m_Analyzer->vratNalezeneKombinace()[i].kostkyKombinace.size(); j++){
                    std::cout<<m_Analyzer->vratNalezeneKombinace()[i].kostkyKombinace[j].hodnota<<" ";
                }
                soucetBodu+= m_Analyzer->vratNalezeneKombinace()[i].body;
                std::cout/*<<std::endl*/<<"\tBody: "<<m_Analyzer->vratNalezeneKombinace()[i].body<<std::endl;
        }
        std::cout<<"Soucet kombinaci: "<<soucetBodu<<std::endl;
        std::cout<<"------------"<<std::endl;
    } else{
        std::cout<<std::endl<<"Zadne kombinace nepadly"<<std::endl<<std::endl;
    }
}

void GameEngine::analyzujStat()
{
    int odlozene =0;
    for (int i=0; i<6; i++)
        if (m_posledniHod[i].hodnota==0)
            odlozene++;

    for (int i=0; i<m_Analyzer->vratNalezeneKombinace().size(); i++)
    {
        m_Statistika->analyzujStat(m_Analyzer->vratNalezeneKombinace()[i].kostkyKombinace.size(), odlozene, m_docasneSkore);
    }
}

void GameEngine::analyzujStat2()
{
    coHrat dalsiTah;
    m_docasneSkore=0;
    int odlozene =0;
    for (int i=0; i<6; i++)
        if (m_posledniHod[i].hodnota==0)
            odlozene++;

    std::vector<sKombinace> kombinace = m_Analyzer->vratNalezeneKombinace();
    dalsiTah= m_Statistika->vyberPostup(m_Analyzer->vratNalezeneKombinace(), odlozene, m_docasneSkore);

    if (dalsiTah.indexKomb == -1)
    {
        std::cout<<"Nepadlo nic, zapisuji nulu."<<std::endl<<std::endl;
    }

    if (dalsiTah.indexKomb==1000)
    {
        if(dalsiTah.hratDal==true)
            std::cout<<"Hraju do plnejch."<<std::endl<<std::endl;
        else
            std::cout<<"Koncim hazeni, beru vse co mam. "<<std::endl<<std::endl;
    }

    if (dalsiTah.indexKomb != 1000 && dalsiTah.indexKomb != -1)
    {
        std::cout<<"Body vybr. kombinace "<<kombinace[dalsiTah.indexKomb].body<<std::endl;
        std::cout<<"Index vybr. kombinace "<<dalsiTah.indexKomb<<std::endl;
        std::cout<<"Hrat dal "<<dalsiTah.hratDal<<std::endl<<std::endl;
    }

}

void GameEngine::fazeHazeni(bool pc)
{
    if (pc) //hraje pocitac
    {
        //inicializace
        coHrat dalsiTah;
        dalsiTah.hratDal= true;
        dalsiTah.indexKomb= 0;
        int odlozene =0;
        m_docasneSkore=0;
        m_AI->nulujOdlozeni();
        m_GUI->resetujVybraneKombinace();
        m_odlozeneCisla.clear();

        while (dalsiTah.hratDal==true && dalsiTah.indexKomb!=-1)
        {
            odlozene=0;
            m_AI->hodKostkama(false);
            naplnPolePosledniHod(m_AI);
            //vypisPolePosledniHod();
            m_Analyzer->analyzujKombinace(m_posledniHod);
            //vypisNalezenychKombinaci();


            for (int i=0; i<6; i++)
            if (m_posledniHod[i].hodnota==0)
                odlozene++;

            std::vector<sKombinace> kombinace = m_Analyzer->vratNalezeneKombinace();
            dalsiTah= m_Statistika->vyberPostup(m_Analyzer->vratNalezeneKombinace(), odlozene, m_docasneSkore);
            m_GUI->zobrazKolo(kombinace, m_odlozeneCisla, m_docasneSkore, true, m_posledniHod, m_PL, m_AI);
            //system("pause"); //CLS az za prvnim IFEM


            if (dalsiTah.indexKomb==1000)
            {

                if(dalsiTah.hratDal==true)
                {
                   //std::cout<<"Hraju do plnejch."<<std::endl<<std::endl;
                   m_GUI->hraDoPlnych();
                   m_AI->nulujOdlozeni();
                   resetujOdlozenaCisla();
                }
                else
                {
                    m_GUI->zobrazAnalyzu(kombinace, m_posledniHod, m_odlozeneCisla, dalsiTah.indexKomb, m_docasneSkore);
                    //std::cout<<"Koncim hazeni, beru vse co mam. "<<std::endl<<std::endl;
                    m_AI->nulujOdlozeni();
                    resetujOdlozenaCisla();
                }

                //sectu vsechny kombinace
                for (int i=0; i<kombinace.size(); i++)
                {
                    m_docasneSkore+= kombinace[i].body;
                }
            }
            system("CLS");
            if (dalsiTah.indexKomb != 1000 && dalsiTah.indexKomb != -1)
            {
                m_GUI->zobrazAnalyzu(kombinace, m_posledniHod, m_odlozeneCisla, dalsiTah.indexKomb, m_docasneSkore);
                //pripisu docasne skore
                m_docasneSkore+= kombinace[dalsiTah.indexKomb].body;
                //zapisi odlozeni do GUI
                zapisDoOdlozenychCisel(kombinace[dalsiTah.indexKomb].kostkyKombinace);
                // projit kombnaci podle index kombinace a vsechny kostky odlozit
                for (int i=0; i<kombinace[dalsiTah.indexKomb].kostkyKombinace.size(); i++)
                {
                    m_AI->odlozKostku(kombinace[dalsiTah.indexKomb].kostkyKombinace[i].poradi);
                }
            }
        }

        resetujOdlozenaCisla();

        if (dalsiTah.indexKomb!=-1)
        {
            m_GUI->pocitacHruKonci();
            system("pause");
            m_AI->pripisSkore(m_docasneSkore);
            m_GUI->vypisKonecKola(m_AI->get_stavSkore() ,m_docasneSkore);
        }
        else
            m_GUI->vypisKonecKola(m_AI->get_stavSkore() ,0);
    }
    else //hazi hrac
    {
        int pocetKostekKombinaci= 0;
        m_docasneSkore= 0;          //docasne skore kola
        m_odlozeneCisla.clear();    //kazde odlozene cislo zapisi sem
        bool pokracovat= true;
        m_PL->nulujOdlozeni();

        while (pokracovat)
        {
            m_PL->hodKostkama(false);
            naplnPolePosledniHod(m_PL);
            m_Analyzer->analyzujKombinace(m_posledniHod);
            std::vector<sKombinace> kombinace = m_Analyzer->vratNalezeneKombinace();

            //nepadlo nic
            if (kombinace.size()==0)
            {
                m_GUI->vypisPanelPrubezneSkore(m_PL, m_AI);
                m_GUI->vypisHodu(m_posledniHod);
                m_GUI->vypisNalezenychKombinaci(kombinace, m_odlozeneCisla, m_docasneSkore, false);
                system("pause");
                m_GUI->vypisKonecKola(m_PL->get_stavSkore(), 0);
                pokracovat= false;
            }

            //neco padlo
            if (kombinace.size()>0)
            {
                pocetKostekKombinaci= 0;
                for (int i=0; i<kombinace.size(); i++)
                {
                    pocetKostekKombinaci +=kombinace[i].kostkyKombinace.size();
                }

                //hra do plnych
                if ((6-pocetKostekKombinaci-m_odlozeneCisla.size())==0)
                {
                    m_GUI->vypisPanelPrubezneSkore(m_PL, m_AI);
                    m_GUI->vypisHodu(m_posledniHod);
                    m_GUI->vypisNalezenychKombinaci(kombinace, m_odlozeneCisla, m_docasneSkore, false);
                    m_GUI->hraDoPlnych();

                    for (int i=0; i<kombinace.size(); i++)
                    {
                        m_docasneSkore +=kombinace[i].body;
                    }

                    m_PL->nulujOdlozeni();
                    pocetKostekKombinaci= 0;
                    m_odlozeneCisla.clear();

                    pokracovat = true;
                }
                else    //normalni stav odkladani
                {
                    pokracovat= m_GUI->zobrazKolo(kombinace, m_odlozeneCisla, m_docasneSkore, false, m_posledniHod, m_PL, m_AI);
                    std::vector<int> vybKomb=  m_GUI->vratVybraneKombinace();

                    if (pokracovat==false)
                    {
                        for (int i=0; i<kombinace.size(); i++)
                        {
                            m_docasneSkore +=kombinace[i].body;
                        }
                        m_PL->pripisSkore(m_docasneSkore);
                        m_GUI->vypisKonecKola(m_PL->get_stavSkore(), m_docasneSkore);
                    }
                    else //pokracovat v hazeni
                    {
                        for (int i=0; i<vybKomb.size(); i++)
                        {
                            //zapisuji body kombinace
                            m_docasneSkore+= kombinace[vybKomb[i]-1].body;
                            //odkladam prislusne kostky
                            for (int j=0; j<kombinace[vybKomb[i]-1].kostkyKombinace.size(); j++)
                            {
                                m_PL->odlozKostku(kombinace[vybKomb[i]-1].kostkyKombinace[j].poradi);
                                m_odlozeneCisla.push_back(kombinace[vybKomb[i]-1].kostkyKombinace[j].hodnota);
                            }
                        }
                    }
                    m_GUI->resetujVybraneKombinace();
                }
            }
        }
        //m_GUI->vypisPanelPrubezneSkore(m_PL, m_AI);
        //m_GUI->vypisHodu(m_posledniHod);
        //m_GUI->vypisNalezenychKombinaci(kombinace, m_odlozeneCisla, m_docasneSkore, false);
        m_PL->nulujOdlozeni();
    }
    m_GUI->resetujVybraneKombinace();
}


bool GameEngine::vyhra()
{
    if (m_AI->get_stavSkore()>=m_EndScore || m_PL->get_stavSkore()>=m_EndScore)
        return true;
    return false;
}

void GameEngine::resetujOdlozenaCisla()
{
    m_odlozeneCisla.clear();
}

void GameEngine::zapisDoOdlozenychCisel(std::vector<sKostkaPoradi> odlozenaKomb)
{
    for (int i=0; i<odlozenaKomb.size(); i++)
    {
        m_odlozeneCisla.push_back(odlozenaKomb[i].hodnota);
    }
}


